# -*- coding: utf-8 -*-

# Copyright 2014-2017 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extract images from https://danbooru.donmai.us/"""

from . import booru


class DanbooruExtractor(booru.JSONBooruExtractor):
    """Base class for danbooru extractors"""
    category = "danbooru"
    api_url = "https://danbooru.donmai.us/posts.json"


class DanbooruTagExtractor(DanbooruExtractor, booru.BooruTagExtractor):
    """Extractor for images from danbooru based on search-tags"""
    subcategory = "tag"
    pattern = [(r"(?:https?://)?(?:www\.)?danbooru.donmai.us/posts"
                r"\?(?:utf8=%E2%9C%93&)?tags=([^&]+)")]
    test = [("https://danbooru.donmai.us/posts?tags=bonocho", {
        "url": "04f69a187269019f6d0c32dc7685246fe99d536e",
        "content": "b196fb9f1668109d7774a0a82efea3ffdda07746",
    })]


class DanbooruPoolExtractor(DanbooruExtractor, booru.BooruPoolExtractor):
    """Extractor for image-pools from danbooru"""
    subcategory = "pool"
    pattern = [r"(?:https?://)?(?:www\.)?danbooru.donmai.us/pools/(\d+)"]
    test = [("https://danbooru.donmai.us/pools/7659", {
        "url": "73c02ec3d67ffd46d489dd6cfd7b35fee053865c",
        "content": "b16bab12bea5f7ea9e0a836bf8045f280e113d99",
    })]


class DanbooruPostExtractor(DanbooruExtractor, booru.BooruPostExtractor):
    """Extractor for single images from danbooru"""
    subcategory = "post"
    pattern = [r"(?:https?://)?(?:www\.)?danbooru.donmai.us/posts/(\d+)"]
    test = [("https://danbooru.donmai.us/posts/294929", {
        "url": "954a9fec4d507c46e220c9beb4349b303b1f4433",
        "content": "5e255713cbf0a8e0801dc423563c34d896bb9229",
    })]



# -*- coding: utf-8 -*-

# Copyright 2015, 2016 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extract manga-chapters and entire manga from http://mangapark.me/"""

from .common import Extractor, Message
from .. import text


class MangaparkMangaExtractor(Extractor):
    """Extractor for mangas from mangapark.me"""
    category = "mangapark"
    subcategory = "manga"
    pattern = [r"(?:https?://)?(?:www\.)?mangapark\.me/manga/([^/]+)$"]
    test = [("http://mangapark.me/manga/mushishi", {
        "url": "9902e342af71af19a5ac20fcd01950b165acf119",
    })]
    url_base = "http://mangapark.me"

    def __init__(self, match):
        Extractor.__init__(self)
        self.url_title = match.group(1)

    def items(self):
        yield Message.Version, 1
        for chapter in self.get_chapters():
            yield Message.Queue, self.url_base + chapter

    def get_chapters(self):
        """Return a list of all chapter urls"""
        page = self.request(self.url_base + "/manga/" + self.url_title).text
        needle = '<a class="ch sts sts_1" target="_blank" href="'
        pos = page.index('<div id="list" class="book-list">')
        return reversed(list(
            text.extract_iter(page, needle, '"', pos)
        ))


class MangaparkChapterExtractor(Extractor):
    """Extractor for manga-chapters from mangapark.me"""
    category = "mangapark"
    subcategory = "chapter"
    directory_fmt = ["{category}", "{manga}",
                     "c{chapter:>03}{chapter-minor} - {title}"]
    filename_fmt = ("{manga}_c{chapter:>03}{chapter-minor}_"
                    "{page:>03}.{extension}")
    pattern = [(r"(?:https?://)?(?:www\.)?mangapark\.me/manga/"
                r"([^/]+/s(\d+)(?:/v([^/]+))?/c(\d+)(?:([^/]+)|/e(\d+))?)")]
    test = [
        ("http://mangapark.me/manga/gosu/s2/c55", {
            "url": "fefe84492d9118de5962563fbecb9362051c52d5",
            "keyword": "652b38c40bdfb5592456b6e7524a3acfdef9fae6",
        }),
        (("http://mangapark.me/manga/"
          "ad-astra-per-aspera-hata-kenjirou/s1/c1.2"), {
            "url": "64b47f9837d50c3e57793ff6703d840ef7808c52",
            "keyword": "f28eb26b4966bebda0e761f241c2dd49e505ce13",
        }),
        ("http://mangapark.me/manga/gekkan-shoujo-nozaki-kun/s2/c70/e2/1", {
            "url": "f8915e25895d4b336892f8a6bd27d26cdb337045",
            "keyword": "34aa6ca3bdf5078f839cbf68ff68e39728cf248b",
        })
    ]

    def __init__(self, match):
        Extractor.__init__(self)
        self.part = match.group(1)
        self.version = match.group(2)
        self.volume = match.group(3)
        self.chapter = match.group(4)
        try:
            self.chminor = match.group(5) or "v" + match.group(6)
        except TypeError:
            self.chminor = ""

    def items(self):
        page = self.request("http://mangapark.me/manga/" + self.part +
                            "?zoom=2").text
        data = self.get_job_metadata(page)
        yield Message.Version, 1
        yield Message.Directory, data
        for url, image in self.get_images(page):
            data.update(image)
            yield Message.Url, url, text.nameext_from_url(url, data)

    def get_job_metadata(self, page):
        """Collect metadata for extractor-job"""
        data = {
            "version": self.version,
            "volume": self.volume or "",
            "chapter": self.chapter,
            "chapter-minor": self.chminor or "",
            "lang": "en",
            "language": "English",
        }
        data = text.extract_all(page, (
            ("manga-id"  , "var _manga_id = '", "'"),
            ("chapter-id", "var _book_id = '", "'"),
            ("manga"     , "<h2>", "</h2>"),
            ("title"     , "</a>", "<"),
            (None        , 'target="_blank" href="', ''),
            ("count"     , 'page 1">1 / ', '<'),
        ), values=data)[0]
        data["manga"], data["type"] = data["manga"].rsplit(" ", maxsplit=1)
        data["manga"] = text.unescape(data["manga"])
        pos = data["title"].find(": ")
        data["title"] = data["title"][pos+2:] if pos != -1 else ""
        return data

    @staticmethod
    def get_images(page):
        """Collect image-urls, -widths and -heights"""
        pos = 0
        num = 0
        while True:
            url, pos = text.extract(page, ' target="_blank" href="', '"', pos)
            if not url:
                return
            num += 1
            width , pos = text.extract(page, ' width="', '"', pos)
            height, pos = text.extract(page, ' _heighth="', '"', pos)
            yield url, {
                "page": num,
                "width": width,
                "height": height,
            }



# -*- coding: utf-8 -*-

# Copyright 2015-2017 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extract images from http://www.hentai-foundry.com/"""

from .common import Extractor, Message
from .. import text, exception


class HentaifoundryUserExtractor(Extractor):
    """Extractor for all images of a hentai-foundry-user"""
    category = "hentaifoundry"
    subcategory = "user"
    directory_fmt = ["{category}", "{artist}"]
    filename_fmt = "{category}_{index}_{title}.{extension}"
    pattern = [
        (r"(?:https?://)?(?:www\.)?hentai-foundry\.com/"
         r"pictures/user/([^/]+)/?$"),
        (r"(?:https?://)?(?:www\.)?hentai-foundry\.com/"
         r"user/([^/]+)/profile"),
    ]
    test = [
        ("http://www.hentai-foundry.com/pictures/user/Tenpura", {
            "url": "dee5fd22cfed3288ce1f637e405781753d7cec57",
            "keyword": "0e47ebc68346087312e62885d9508fef7ad55503",
        }),
        ("http://www.hentai-foundry.com/user/asdq/profile", {
            "exception": exception.NotFoundError,
        }),
    ]
    url_base = "http://www.hentai-foundry.com/pictures/user/"

    def __init__(self, match):
        Extractor.__init__(self)
        self.artist = match.group(1)

    def items(self):
        data, token = self.get_job_metadata()
        self.set_filters(token)
        yield Message.Version, 1
        yield Message.Directory, data
        for url, image in self.get_images():
            image.update(data)
            yield Message.Url, url, image

    def get_images(self):
        """Yield url and keywords for all images of one artist"""
        num = 1
        needle = 'thumbTitle"><a href="/pictures/user/'
        while True:
            pos = 0
            url = self.url_base + self.artist + "/page/" + str(num)
            page = self.request(url).text
            for _ in range(25):
                part, pos = text.extract(page, needle, '"', pos)
                if not part:
                    return
                yield self.get_image_metadata(self.url_base + part)
            num += 1

    def get_job_metadata(self):
        """Collect metadata for extractor-job"""
        url = self.url_base + self.artist + "?enterAgree=1"
        response = self.session.get(url)
        if response.status_code == 404:
            raise exception.NotFoundError("user")
        page = response.text
        token, pos = text.extract(page, 'hidden" value="', '"')
        count, pos = text.extract(page, 'class="active" >Pictures (', ')', pos)
        return {"artist": self.artist, "count": count}, token

    def get_image_metadata(self, url):
        """Collect metadata for an image"""
        page = self.request(url).text
        offset = len(self.url_base) + len(self.artist)
        index = text.extract(url, '/', '/', offset)[0]
        title, pos = text.extract(
            page, 'Pictures</a> &raquo; <span>', '<')
        url, pos = text.extract(
            page, '//pictures.hentai-foundry.com', '"', pos)
        data = {"index": index, "title": text.unescape(title)}
        text.nameext_from_url(url, data)
        return "http://pictures.hentai-foundry.com" + url, data

    def set_filters(self, token):
        """Set site-internal filters to show all images"""
        formdata = {
            "YII_CSRF_TOKEN": token,
            "rating_nudity": 3,
            "rating_violence": 3,
            "rating_profanity": 3,
            "rating_racism": 3,
            "rating_sex": 3,
            "rating_spoilers": 3,
            "rating_yaoi": 1,
            "rating_yuri": 1,
            "rating_teen": 1,
            "rating_guro": 1,
            "rating_furry": 1,
            "rating_beast": 1,
            "rating_male": 1,
            "rating_female": 1,
            "rating_futa": 1,
            "rating_other": 1,
            "filter_media": "A",
            "filter_order": "date_new",
            "filter_type": 0,
        }
        self.request("http://www.hentai-foundry.com/site/filters",
                     method="post", data=formdata)


class HentaifoundryImageExtractor(Extractor):
    """Extractor for a single image from hentaifoundry.com"""
    category = "hentaifoundry"
    subcategory = "image"
    directory_fmt = ["{category}", "{artist}"]
    filename_fmt = "{category}_{index}_{title}.{extension}"
    pattern = [(r"(?:https?://)?(?:www\.|pictures\.)?hentai-foundry\.com/"
                r"(?:pictures/user/([^/]+)/(\d+)"
                r"|[^/]/([^/]+)/(\d+))")]
    test = [
        (("http://www.hentai-foundry.com/"
          "pictures/user/Tenpura/340854/notitle"), {
            "url": "f3c0739bf86543697deabbed4bf99eb95a04582b",
            "keyword": "96217c5becc1369c36dafa201c3c208518de8f1f",
            "content": "5c14cd10deaad79a5152f9de45c9203cf76165a0",
        }),
        ("http://www.hentai-foundry.com/pictures/user/Tenpura/340853/", {
            "exception": exception.NotFoundError,
        }),
    ]

    def __init__(self, match):
        Extractor.__init__(self)
        self.artist = match.group(1) or match.group(3)
        self.index = match.group(2) or match.group(4)

    def items(self):
        url, data = self.get_image_metadata()
        yield Message.Version, 1
        yield Message.Directory, data
        yield Message.Url, url, data

    def get_image_metadata(self):
        """Collect metadata for an image"""
        url = "http://www.hentai-foundry.com/pictures/user/{}/{}".format(
            self.artist, self.index)
        response = self.session.get(url + "?enterAgree=1")
        if response.status_code == 404:
            raise exception.NotFoundError("image")
        extr = text.extract
        page = response.text
        artist, pos = extr(page, '<a href="/pictures/user/', '"')
        title , pos = extr(page, 'Pictures</a> &raquo; <span>', '<', pos)
        url   , pos = extr(page, '//pictures.hentai-foundry.com', '"', pos)
        data = {
            "artist": artist,
            "index": self.index,
            "title": text.unescape(title),
        }
        text.nameext_from_url(url, data)
        return "http://pictures.hentai-foundry.com" + url, data



# -*- coding: utf-8 -*-

# Copyright 2017 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extract manga-chapters from https://gomanga.co/"""

from . import foolslide


class GomangaChapterExtractor(foolslide.FoolslideChapterExtractor):
    """Extractor for manga-chapters from gomanga.co"""
    category = "gomanga"
    pattern = foolslide.chapter_pattern(r"(?:www\.)?gomanga\.co/reader")
    test = [
        ("https://gomanga.co/reader/read/mata-kata-omou/en/0/1/page/11", {
            "url": "5088d75bb44327fc503c85b52b1d6a371b8057f2",
            "keyword": "f534cfc4c3aef87cb0439e2a37e2ebee98077e92",
        }),
        ("https://gomanga.co/reader/read/pastel/en/31/144/", {
            "url": "9cc2052fbf36344c573c754c5abe533a14b3e280",
            "keyword": "a2ef55d26984c64baf026382f889bb013d01dc4f",
        }),
    ]
    single = False



# -*- coding: utf-8 -*-

# Copyright 2017 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extract manga-chapters from https://reader.kireicake.com/"""

from . import foolslide


class KireicakeChapterExtractor(foolslide.FoolslideChapterExtractor):
    """Extractor for manga-chapters from reader.kireicake.com"""
    category = "kireicake"
    pattern = foolslide.chapter_pattern("reader\.kireicake\.com")
    test = [("https://reader.kireicake.com/read/wonderland/en/1/1/", {
        "url": "b2d36bc0bc67e4c461c3a4d6444a2fd339f5d07e",
        "keyword": "17d04e3bb24f6ad593463ecb7f90667f0df5326f",
    })]
